// COPYRAND v 0.3b (c) 1998 Graham Mitchell <mitchell@onr.com> - 14 Feb 98
// Copies a random file from the specified location.

/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License (LICENSE.TXT) for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <dir.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>

// written by me, includes functions fnexpand(), exist(), and copyfile()
#include "files.cpp"

// uncomment the following lines if bool is not a built-in type
/*
typedef short bool;
const short false = 0;
const short true = 1;
*/

const int NUM_FILES = 255;
extern const char* helptext;		// help text is listed after main()

char filenames[NUM_FILES][13];

int buildtable(const char* filespec);
static void seedrand();

int main(int argc, char *argv[])
{
	char check = 'n';
	bool overwrite_ok = false;
	int errlvl = 1, srcflags, destflags;
	int FIRST = 1, SECOND = 2;
	int files = 0, filenum = 0;
	char *srcdir, *srcfn;
	char *srcspec, *destfn, *srcpath, *srcfrag;

	seedrand();

	cout << "COPYRAND v 0.3b (c) 1998 Graham Mitchell <mitchell@onr.com>\n";

	if ( (argc != 3) && (argc != 4) )
	{
		cout << helptext;
		return 0;
	}

	else 
		cout << endl;

	if ( argc == 4 )
	{
		// we have an option, let's check if it's /Y

		if ( argv[1][0] == '/' && ( argv[1][1] == 'y' || argv[1][1] == 'Y' ) )
		{
			// it's the first argument, adjust parameter positions

			FIRST = 2;
			SECOND = 3;
			overwrite_ok = true;
		}
		else if ( argv[3][0] == '/' && (argv[3][1]=='y' || argv[3][1]=='Y') )
		{
			// it's the last argument

			overwrite_ok = true;
		}
		else
		{
			cout << "Switch not recognized\n";
			return 1;
		}
	}

	// get source filespec and dest filename from command line
	srcdir = argv[FIRST];
	srcspec = new char[255];
	srcflags = fnexpand(srcspec, srcdir);
	destfn = new char[255];
	destflags = fnexpand(destfn, argv[SECOND]);

	// Check to make sure destination doesn't contain wildcards

	if ( destflags&WILDCARDS )
	{
		cout << "Cannot copy to multiple destinations\n\t0 file(s) copied\n";
		return 1;
	}

	// At this point, srcspec has the location to pick the random file from,
	//    and destfn has the full-blown name of the destination

	// build table of filenames
	if ( !(srcflags&FILENAME) )
	{
		// source is a directory, use *.* as the filespec
		strcat(srcspec, "*.*");
	}

	files = buildtable(srcspec);

	cout << files << " file(s) match";

	if ( files == 0 )
	{
		cout << "\n\t0 file(s) copied\n";
		return 1;
	}	

	filenum = rand() % files;

	srcfn = filenames[filenum];

	// generate a path name for the source file
	srcfrag = new char[255];
	char drive[_MAX_DRIVE], dir[_MAX_DIR];
	char name[_MAX_FNAME], ext[_MAX_EXT];

	_splitpath(srcspec, drive, dir, name, ext);
	_makepath(srcfrag, drive, dir, NULL, NULL);

	strcat(srcfrag, srcfn);

	srcpath = new char[255];
	
	fnexpand(srcpath, srcfrag);

	cout << ": copying " << srcfn << endl;

	// check for source == dest

	if ( strcmp(srcpath,destfn) == 0 )
	{
		cout << "File cannot be copied onto itself\n\t0 file(s) copied\n";
		return 1;
	}

	// if the destination is a directory, use the source filename

	if ( !(destflags&FILENAME) )
		strcat(destfn,srcfn);

	// Finally, check to see if destination exists before we copy anything

	if ( !overwrite_ok && exist(destfn) )
	{
		cout << "Overwrite " << destfn << " (Yes/No)?";
		cin >> check;
		if ( (check != 'y') && (check !='Y') )
		{
			cout << "\t0 file(s) copied\n";
			return 0;
		}
	} 

	// if we've gotten this far, it's okay to overwrite the destination
	//    or it doesn't exist (same difference to us)

	errlvl = copyfile(srcpath,destfn);

	if ( errlvl == 0 )
		cout << "\t1 file(s) copied\n";
	else
	{
		cout << "\t0 file(s) copied\n";
		return 1;
	}

	return 0;
}

const char* helptext = \
"This program is free software, protected by the GNU General Public \
License.\n\nCopies random file to another location.\n\nCOPYRAND [/Y] \
source destination\n\n  source\tSpecifies the source directory and/or \
filespec.\n  destination\tspecifies the directory and/or filename for the \
new file.\n  /Y\t\tSuppresses prompting to confirm you want to overwrite \
an\n  \t\texisting destination file.\n\nEXAMPLES:\n\n\tCOPYRAND *.BMP \
CURRENT.BMP\n\nRandomly chooses a .BMP file from the current directory and \
copies it\nto CURRENT.BMP in the same directory\n\n\tCOPYRAND \
C:\\FORTUNES\\*.* D:\\BBS\\MOTD.TXT\n\nPicks a file from C:\\FORTUNES and \
copies it to the file MOTD.TXT in D:\\BBS\n"; \

// buildtable()
//
// Uses file control blocks to generate a table of files matching the
//	given filespec.  Returns the total number of matching files
// requires DIR.H, STRING.H
//
int buildtable(const char* filespec)
{
	struct ffblk fileblock;
	int done;
	int filenum = 0;

	done = findfirst(filespec,&fileblock,0);
	while (!done)
	{
		strcpy(filenames[filenum], fileblock.ff_name);
		filenum++;
		done = findnext(&fileblock);
	}

	return filenum;
}

// seedrand()
//
// Seeds the random number generator.  Differs from randomize()
//	in that it seeds based on hundredths of seconds, not seconds
// requires DOS.H, STDLIB.H
//
static void seedrand()
{
	struct dostime_t t;
	_dos_gettime(&t);
	srand(t.hsecond);
}

